@@ -17,117 +17,98 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Fields.py,v 1.5 1999/07/30 12:32:12 gobry Exp $
+# $Id: Fields.py,v 1.9 1999/08/18 14:05:16 gobry Exp $
 
-import string
+import string, types, re, string
 
-# Available field types
-TypeText   = 0
-TypeAuthor = 1
-TypeTitle  = 2
-TypeDate   = 3
+year_match = re.compile ('(\d\d\d\d)')
 
-class Description:
-    """ Generic Holder for information related to Field/Entry descriptions """
+class Access:
+    """ virtual class used to check if instance attributes have been
+    modified """
     
-    def __init__ (self, name):
-        self.name  = name
-        self.__items__ = {}
-        
-    def __getattr__ (self, attr):
-        raise AttributeError, "no attribute `%s' in Description" % (attr)
+    def __init__ (self):
+        self.clean ()
+        return
 
     def __setattr__ (self, attr, value):
+        if attr == '_modified_':
+            raise AttributeError, "can't modify _modified_"
+        
         self.__dict__ [attr] = value
+        self.touch ()
         return
-    
-    def __getitem__ (self, item):
-        return self.__items__ [item]
 
-    def __setitem__ (self, item, value):
-        self.__items__ [item] = value
+    def modified (self):
+        return self._modified_
+
+    def clean (self):
+        self.__dict__ ['_modified_'] = 0
         return
-    
-    def has_key (self, field):
-        return self.__items__.has_key (field)
 
-    
-class FieldDescription (Description):
-    """ Available informations for a given field type """
-    
-    def __init__ (self, name, type = TypeText):
-        
-        Description.__init__ (self, name)
-        self.type = type
-        
-    
-class EntryDescription (Description):
-    """ Informations on a given entry type """
-    
-    def __init__ (self, name):
-        Description.__init__ (self, name)
-        
-        self.__dict__ ['mandatory'] = []
-        self.__dict__ ['optional']  = []
-        self.__dict__ ['lcfields']  = {}
-        
-    def __getattr__ (self, attr):
-        if attr == 'fields':
-            return self.mandatory + self.optional
-        
-        return Description.__getattr__ (self, attr)
-
-    def __setattr__ (self, attr, value):
-
-        if attr == 'mandatory' or attr == 'optional':
-            self.__dict__ [attr] = value
-            self.__dict__ ['lcfields'] = {}
-            
-            for f in self.fields:
-                self.__dict__ ['lcfields'] [string.lower (f.name)] = f
-
-        else:
-            Description.__setattr__ (self, attr, value)
-
-    def __getitem__ (self, item):
-        return self.__dict__ ['lcfields'] [string.lower (item)]
-
-    def __setitem__ (self, item, value):
-        self.__dict__ ['lcfields'] [string.lower (item)] = value
-
-    def has_key (self, field):
-        return self.__dict__ ['lcfields'].has_key (string.lower (field))
+    def touch (self):
+        self.__dict__ ['_modified_'] = 1
+        return
 
         
-class Author:
+class Author (Access):
     """ Fine description of an author """
     
     def __init__ (self, copy = None):
         """ Initialize an author from a string """
 
-        if copy:
-            self.honorific = copy [0]
-            self.first = copy [1]
-            self.last = copy [2]
-            self.lineage = copy [3]
-        else:
+        if type (copy) is types.StringType:
+            # manually split the author into subfields.
             self.honorific = None
-            self.first = None
-            self.last = None
-            self.lineage = None
+            self.lineage   = None
+
+            blocs = string.split (copy, ',')
+            if len (blocs) == 1:
+                # last name is after lowercase or is last word
+                words = map (string.strip, string.split (blocs [0]))
+                i = 0
+                while i < len (words) - 1:
+                    if words [i] == string.lower (words [i]): break
+                    i = i + 1
+                    
+                self.first = string.join (words [:i], ' ')
+                self.last  = string.join (words [i:], ' ')
+                
+            elif len (blocs) == 2:
+                self.last  = string.strip (blocs [0])
+                self.first = string.strip (blocs [1])
+            else:
+                self.last  = copy
+                self.first = None
+        else:
+            if copy:
+                self.honorific = copy [0]
+                self.first     = copy [1]
+                self.last      = copy [2]
+                self.lineage   = copy [3]
+            else:
+                self.honorific = None
+                self.first     = None
+                self.last      = None
+                self.lineage   = None
 
         self.text = None
-        
+
+        Access.__init__ (self)
+        return
+    
     def __str__ (self):
         """ Returns textual representation """
-        
-        if not self.text:
+
+        changed = self.modified ()
+        if changed or not self.text:
             text = ""
             if self.honorific: text = text + " " + self.honorific
             if self.first:     text = text + " " + self.first
             if self.last:      text = text + " " + self.last
             if self.lineage:   text = text + " " + self.lineage
             self.text = text [1:]
+            if not changed: self.clean ()
             
         return self.text
 
@@ -167,21 +148,34 @@ class Author:
         return 0
 
 
-class AuthorGroup:
+class AuthorGroup (Access):
     """ A group of Authors """
 
     def __init__ (self):
         self.authors = []
+        Access.__init__ (self)
+        return
 
     def __getitem__ (self, pos):
         return self.authors [pos]
 
     def __setitem__ (self, pos, val):
         self.authors [pos] = val
-        return self.authors [pos]
+        self.val.touch ()
+        return
 
     def __len__ (self):
         return len (self.authors)
+
+    def modified (self):
+        for a in self.authors:
+            if a.modified (): return 1
+        return 0
+
+    def clean (self):
+        for a in self.authors:
+            a.clean ()
+        return
     
     def append (self, value):
         self.authors.append (value)
@@ -197,7 +191,11 @@ class AuthorGroup:
 
     def __cmp__ (self, other):
         i = 0
-        s = len (self), len (other)
+        try:
+            s = len (self), len (other)
+        except TypeError:
+            return 1
+        
         m = max (s)
         
         while i < m:
@@ -213,10 +211,25 @@ class AuthorGroup:
             
 RangeError = "RangeError"
 
-class Date:
+class Date (Access):
     """ Fine description of a date """
 
-    def __init__ (self, year = None, month = None, day = None):
+    def __init__ (self, arg = (None, None, None)):
+
+        if type (arg) is types.StringType:
+            try:
+                year  = int (arg)
+            except ValueError:
+                g = year_match.search (arg)
+                if g:
+                    year = int (g.group (1))
+                else:
+                    raise ValueError, "can't parse `%s' as a date" % arg
+                
+            month = None
+            day   = None
+        else:
+            year, month, day = arg
         
         if year and year < 0:
             raise RangeError, "wrong year"
@@ -231,8 +244,11 @@ class Date:
         self.day = day
 
         self.text = None
+        Access.__init__ (self)
+        return
         
     def __cmp__ (self, other):
+
         # No date is no date !
         if not self.year and not other.year: return 0
 
@@ -252,15 +268,18 @@ class Date:
 
     def __str__ (self):
         """ Returns textual representation """
-        
-        if self.year and self.month and self.day:
-            self.text = "%d/%d/%d" % (self.day, self.month, self.year)
+        changed = self.modified ()
 
-        if self.year and self.month:
-            self.text = "%d/%d" % (self.month, self.year)
+        if changed or not self.text:
+            if self.year and self.month and self.day:
+                self.text = "%d/%d/%d" % (self.day, self.month, self.year)
+
+            if self.year and self.month:
+                self.text = "%d/%d" % (self.month, self.year)
+            
+            self.text = str (self.year)
+            if not changed: self.clean ()
             
-        self.text = str (self.year)
-
         return self.text
 
 
@@ -273,16 +292,21 @@ class Date:
         return regex.search (str (self))
         
 
-class Text:
+class Text (Access):
+    """ Content of all the other fields """
+    
     def __init__ (self, text):
         self.text = text
 
+        Access.__init__ (self)
+        return
+
 
     def __str__ (self):
         return str (self.text)
 
 
-    def __repr (self):
+    def __repr__ (self):
         return `self.text`
 
 
@@ -292,9 +316,5 @@ class Text:
 
 
     def __cmp__ (self, other):
-        try:
-            return cmp (string.lower (self.text), string.lower (other.text))
-        except AttributeError:
-            print `self`
-            print `other`
-            raise AttributeError
+        return cmp (self.text, str (other))
+
