@@ -17,23 +17,29 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: BibTeX.py,v 1.17 1999/07/30 12:34:00 gobry Exp $
+# $Id: BibTeX.py,v 1.23 1999/08/18 14:05:21 gobry Exp $
 
 # Extension module for BibTeX files
 
-import Pyblio.Base, bibtex
+import Pyblio.Base, _bibtex
 import os,sys, tempfile, pwd, time, traceback, re, string
 import Pyblio.Open
 
 from types import *
 from Pyblio.Fields import *
+from Pyblio import Types
+
+def _nativify (field, type):
+	obj = _bibtex.reverse (type, field)
+	return _bibtex.native (obj)
 
 
-def my_write (entry, output):
+def entry_write (entry, output):
 	""" Print an entry as BiBTeX """
 
 	native = (entry.id == 'BibTeX')
 
+
 	# get the entry description
 	tp = entry.type
 	
@@ -45,9 +51,10 @@ def my_write (entry, output):
 		
 		if entry.has_key (field):
 			if native:
-				text = entry.native (field)
+				text = entry.get_native (field)
 			else:
-				text = entry [field]
+				type = Types.gettype (tp, field)
+				text = _nativify (entry [field], type)
 				
 			output.write ('  ' + Pyblio.Base.format (
 				'%-14s = %s' % (f.name, text),
@@ -56,17 +63,55 @@ def my_write (entry, output):
 
 	for f in dico:
 		if native:
-			text = entry.native (f)
+			text = entry.get_native (f)
 		else:
-			text = entry [f]
+			type = Types.gettype (tp, f)
+			text = _nativify (entry [f], type)
 			
 		output.write ('  ' + Pyblio.Base.format (
 			'%-14s = %s' % (f, text), 75, 0, 19) +
                               ',\n')
 		
         output.write ('}\n\n')
+	return
 
 
+def my_write (database, output):
+	need_header = []
+	if database.id == 'BibTeX':
+		need_header = [database]
+	else:
+		# ugly hack to get the BibTeX databases involved
+		db = {}
+		def check_entry (entry, need, database = database):
+			# guess what, we are in a RefDB !
+			if entry.id == 'BibTeX':
+				base = database.base (entry)
+				# ensure unicity
+				need [base.key] = base
+			return
+		database.foreach (check_entry, db)
+
+		need_header = db.values ()
+
+	if len (need_header) > 0:
+		for db in need_header:
+			parser = db.get_parser ()
+		
+			# write the list of strings
+			dict = _bibtex.get_dict (parser)
+		
+			if len (dict.keys ()) > 0:
+				output.write ('@string{\n')
+				for k in dict.keys ():
+					value = _bibtex.native (dict [k])
+					output.write ("  %s \t= %s,\n" % (k, value))
+				output.write ('}\n\n')
+
+	# write all the entries
+	database.foreach (entry_write, output)
+	return
+
 # --------------------------------------------------
 # Register a method to open BibTeX files
 # --------------------------------------------------
@@ -82,10 +127,10 @@ def my_open (entity, check):
 	return base
 
 
+Pyblio.Open.register ('BibTeX', {'open'  : my_open,
+				 'write' : my_write })
 
-Pyblio.Open.register ('BibTeX', my_open, my_write)
-
-
+	
 # --------------------------------------------------
 #  Entry inherits from pyb
 # --------------------------------------------------
@@ -93,11 +138,12 @@ Pyblio.Open.register ('BibTeX', my_open, my_write)
 class Entry (Pyblio.Base.Entry):
 
 	id = 'BibTeX'
-	
-	def __init__ (self, key, name, type, content):
-		Pyblio.Base.Entry.__init__ (self, key, name, type, content)
 
-		self.__text   = {}
+	def __init__ (self, key, name, type, content, parser):
+		Pyblio.Base.Entry.__init__ (self, key, name, type, content)
+		self.__text = {}
+
+		self.parser = parser
 		return
 	
 	
@@ -108,13 +154,44 @@ class Entry (Pyblio.Base.Entry):
 
 		del self.__dict [key]
 		return
-	
-	def native (self, key):
-		""" Return object in its native format """
-		obj = self.__dict [key]
-		
-		return bibtex.native (obj)
 
+	def __setitem__ (self, key, value):
+		# First, eventually remove from cache
+		if self.__text.has_key (key):
+			del self.__text [key]
+
+		type = Types.gettype (self.type, key)
+		self.__dict [key] = _bibtex.reverse (type, value)
+		return
+
+	
+	def get_native (self, key):
+		""" Return object in its native format """
+
+		if self.__text.has_key (key):
+			obj = self.__text [key] [0]
+			if obj.modified ():
+				type = Types.gettype (self.type, key)
+				self.__dict [key] = \
+					    _bibtex.reverse (type, obj)
+				# update cache
+				del self.__text [key]
+			
+		obj = self.__dict [key]
+		return _bibtex.native (obj)
+	
+
+	def set_native (self, key, value):
+		""" set in native format """
+		
+		if self.__text.has_key (key):
+			del self.__text [key]
+
+		type = Types.gettype (self.type, key)
+		self.__dict [key] =  _bibtex.reverse (type, value)
+		return
+	
+		
 	def text (self, key):
 		# look in the cache first
 		if self.__text.has_key (key):
@@ -124,25 +201,26 @@ class Entry (Pyblio.Base.Entry):
 
 		# search its declared type
 
-		type = Pyblio.Base.gettype (self.type, key)
+		type = Types.gettype (self.type, key)
+		ret = _bibtex.expand (self.parser, obj, type)
 		
-		ret = bibtex.expand (self.key.base.get_parser (),
-				     obj, type)
-		
-		if ret [0] == Pyblio.Fields.TypeAuthor:
+		if ret [0] == Types.TypeAuthor:
 			# Author
 			val = AuthorGroup ()
-			for aut in ret [3]: val.append (Author (aut))
+			for aut in ret [3]:
+				val.append (Author (aut))
 			
-		elif ret [0] == Pyblio.Fields.TypeDate:
+		elif ret [0] == Types.TypeDate:
 			# Date
-			val = Date (ret [3])
+			val = Date ((ret [3], None, None))
 			
 		else:
 			# Any other text
 			val = Text (ret [2])
 
+		val.clean ()
 		self.__text [key] = (val, ret [1])
+		
 		return (val, ret [1])
 
 
@@ -154,20 +232,26 @@ class Entry (Pyblio.Base.Entry):
 
 class DataBase (Pyblio.Base.DataBase):
 
-	id = 'BibTeXDB'
+	id = 'BibTeX'
+	
+	properties = {
+		'edit'        : 1,
+		'change_id'   : 1,
+		'change_type' : 1,
+		'add'         : 1,
+		'remove'      : 1,
+		'has_extra'   : 1,
+		}
 	
 	def __parsefile__ (self):
-		self.__entry = {}
-		self.__parser = 0
+		self.__parser = None
 		self.__recursive = 0
 		self.__current = 0
 		
-		# Stockage temporaire avant un "update"
-		self.__to_delete = {}
-		self.__to_modify = {}
+		self.__dict = {}
 		
 		# Ouvrir le fichier associe
-		self.__parser = bibtex.open (self.name)
+		self.__parser = _bibtex.open (self.name)
 		
 		finished = 0
 		errors = []
@@ -175,7 +259,7 @@ class DataBase (Pyblio.Base.DataBase):
 		# Creer la base de cles
 		while 1:
 			try:
-				retval = bibtex.next (self.__parser)
+				retval = _bibtex.next (self.__parser)
 			except IOError, err:
 				errors.append (str (err))
 				continue
@@ -183,9 +267,13 @@ class DataBase (Pyblio.Base.DataBase):
 			if retval == None: break
 
 			name, type, offset, line, object = retval
+			
+			key = Pyblio.Base.Key (self, name)
 
-			if not self.__entry.has_key (name):
-				self.__entry [name] = offset
+			if not self.__dict.has_key (key):
+				type = Types.getentry (type)
+				entry = Entry (key, name, type, object, self.__parser)
+				self.__dict [key] = entry
 				continue
 				
 			errors.append ("%s:%d: key `%s' already defined" % (
@@ -208,127 +296,14 @@ class DataBase (Pyblio.Base.DataBase):
 	def get_parser (self):
 		return self.__parser
 	
-	def keys (self):
-		""" Return the key sequence """
-		
-		return map (lambda k, self=self:
-			    Pyblio.Base.Key (self, k), self.__entry.keys ())
-
-
-	def __len__ (self):
-		""
-		return len (self.__entry)
 
 	def __repr__ (self):
 		""
 		return "<BibTeX database `%s' (%d entries)>" % \
 		       (self.name, len (self))
-	
-	def has_key (self, key):
-		"Presence d'une cle"
-		return self.__entry.has_key (key.key)
-
-	def __getitem__ (self, name):
-		"Retourne une entree en fonction de la cle"
-		name = name.key
-		
-		type = __builtins__['type']
-
-		# If we get a modified instance, return it directly
-		ref = self.__entry [name]
-
-		if type (ref) is InstanceType:
-			return ref
-		else:
-			# Else read from file
-			bibtex.set_offset (self.__parser, ref)
-
-			name, type, offset, line, content = \
-			      bibtex.next (self.__parser)
-
-			type = Pyblio.Base.getentry (type)
-			
-			# If in recursive mode, go back to previous position
-			if self.__recursive > 0:
-				bibtex.set_offset (self.__parser,
-						   self.__current)
-				
-			return Entry (Pyblio.Base.Key (self, name),
-				      name, type, content);
-
-	
-	def __setitem__ (self, key, value):
-		"Modifier ou ajouter une entree"
-
-		value.name = key.key
-		value.key  = key
-		
-		self.__entry [key.key] = value
-
-		# Sauver la version modifiee aussi
-		self.__to_modify [key.key] = value
-
-		
-	def __delitem__ (self, key):
-		"Detruire une entree"
-
-		if self.has_key (key):
-			# Noter la destruction
-			self.__to_delete [key.key] = 1
-			# Enlever du dictionnaire
-			del self.__entry [key.key]
-			
-		else:
-			raise KeyError, "no such entry"
-
 
 	# ==================================================
 
-	def foreach (self, function, args = None):
-		"Parcours de toutes les entrees"
-		bibtex.first (self.__parser)
-		finished = 0
-
-		# If we are in a recursive state
-		if self.__recursive > 0:
-			previous_position = self.__current
-			
-		# On est susceptible d'avoir un comportement recursif
-		self.__recursive = self.__recursive + 1
-		
-		# Parcourir les entrees du fichier
-		while 1:
-			entry = bibtex.next (self.__parser)
-			
-			if entry == None: break
-
-			name, type, offset, line, content = entry
-
-			type = Pyblio.Base.getentry (type)
-
-			if self.__to_delete.has_key (name):
-				continue
-				
-			if self.__to_modify.has_key (name):
-				continue
-
-			# Save the current position
-			self.__current =  bibtex.get_offset (self.__parser)
-				
-			function (Entry (Pyblio.Base.Key (self, name),
-					 name, type, content), args)
-
-		# Les nouvelles ou les modifiees
-		for entry in self.__to_modify.keys ():
-			function (self.__to_modify [entry], args)
-			
-		# Leave recursive state
-		self.__recursive = self.__recursive - 1
-		if self.__recursive > 0:
-			bibtex.set_offset (self.__parser, previous_position)
-			self.__current = previous_position
-			
-
 	def update (self):
 		""" save the database """
 
@@ -336,30 +311,10 @@ class DataBase (Pyblio.Base.DataBase):
 		os.rename (self.name, self.name + '.bak')
 		
 		tmpfile = open (self.name, 'w')
-		self.foreach (my_write, tmpfile)
+		my_write (self, tmpfile)
 		tmpfile.close ()
 
 		self.__parsefile__ ()
 		return
 
 	
-	def where (self, key):
-		"Chercher une entree selon des criteres"
-		result = []
-
-		def search_function (entry, param):
-			"the real search function "
-			key, result = param
-
-			if key.match (entry):
-				result.append (entry.key)
-						
-		self.foreach (search_function, (key, result))
-
-		# Creer la base de references
-		refs = Pyblio.Base.Reference ()
-
-		if len (result) > 0:
-			refs.add (self, result)
-
-		return refs
