@@ -17,33 +17,117 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Pybliographic.py,v 1.4 1999/07/30 12:34:14 gobry Exp $
+# $Id: Pybliographic.py,v 1.11 1999/08/18 14:05:33 gobry Exp $
+
+import string, os
 
-from Pyblio.Base import *
-import string, os.path
 from gnome.ui import *
 from gtk import *
 import GtkExtra, GDK, GTK
 
-from Pyblio.Open import bibopen, bibwrite
+from Pyblio import Open, Types, Base, Config
+
+from Pyblio.GnomeUI import Utils
+
 from Pyblio.GnomeUI.Database import *
-from Pyblio.GnomeUI.Entry import *
+from Pyblio.GnomeUI.Entry  import *
 from Pyblio.GnomeUI.Search import *
     
-def popup_add (menu, item, action, argument = None):
-    """ Add a new menu entry """
+def popup_add (menu, item, action = None, argument = None):
+    """ Helper to add a new menu entry """
     tmp = GtkMenuItem (item)
-    tmp.connect ("activate", action, argument)
-    
+    if action:
+        tmp.connect ("activate", action, argument)
     tmp.show ()
     menu.append (tmp)
+    
+    return tmp
 
-class Pybliographic (GnomeApp):
 
-    def __init__ (self):
-        GnomeApp.__init__ (self, "Pybliographer", "Pybliographer")
+class URLFileSelection (GtkFileSelection):
+    """ Extended file selection dialog, with an URL field and a type
+    selector. """
+    
+    def __init__(self, title = "File", url=TRUE, modal=TRUE, has_auto=TRUE):
+        
+        GtkFileSelection.__init__(self)
+        self.set_title (title)
+        
         self.connect("destroy", self.quit)
         self.connect("delete_event", self.quit)
+        self.cancel_button.connect('clicked', self.quit)
+        self.ok_button.connect('clicked', self.ok_cb)
+
+        if modal:
+            grab_add(self)
+
+        self.ret = None
+
+        vbox = self.main_vbox
+        
+        # url handler
+        hbox = GtkHBox ()
+        hbox.set_spacing (5)
+        hbox.pack_start (GtkLabel ("URL:"), expand = FALSE, fill = FALSE)
+        self.url = GtkEntry ()
+        hbox.pack_start (self.url)
+        vbox.pack_start (hbox, expand = FALSE, fill = FALSE)
+
+        # type selector
+        hbox = GtkHBox ()
+        hbox.set_spacing (5)
+        hbox.pack_start (GtkLabel ("Bibliography type:"),
+                         expand = FALSE, fill = FALSE)
+        self.menu = GtkOptionMenu ()
+        hbox.pack_start (self.menu)
+        vbox.pack_start (hbox, expand = FALSE, fill = FALSE)
+
+        # menu content
+        menu = GtkMenu ()
+        self.menu.set_menu (menu)
+        if has_auto:
+            popup_add (menu, " - Auto - ", self.menu_select, None)
+
+        for avail in Open.available ():
+            popup_add (menu, avail, self.menu_select, avail)
+            
+        self.menu.set_history (0)
+        self.type = None
+        return
+
+    def menu_select (self, widget, selection):
+        self.type = selection
+        return
+        
+    def quit(self, *args):
+        self.hide()
+        self.destroy()
+        mainquit()
+        return
+    
+    def ok_cb(self, b):
+        self.ret = self.get_filename()
+        if self.ret [-1] == '/':
+            self.ret = self.url.get_text ()
+            if self.ret == "": self.ret = None
+        self.quit()
+        return
+    
+    def run (self):
+        self.show_all ()
+        mainloop ()
+        
+        return self.ret
+
+    
+class Pybliographic (GnomeApp):
+    """ Main Class defining a Pybliographic window """
+    
+    def __init__ (self):
+        GnomeApp.__init__ (self, "Pybliographer", "Pybliographer")
+
+        self.connect ("destroy", self.quit)
+        self.connect ("delete_event", self.quit)
 
         self.set_title("Pybliographic")
         
@@ -121,6 +205,7 @@ class Pybliographic (GnomeApp):
         holder.show ()
 
         self.database.connect ("button_press_event", self.popup_menu)
+        self.database.connect ("key_press_event", self.key_handler)
         self.database.connect ("select_row", self.update_display)
         
         paned.add1 (holder)
@@ -148,9 +233,10 @@ class Pybliographic (GnomeApp):
 
         # Le popup menu
         self.menu = GtkMenu ()
-        popup_add (self.menu, "New...",  self.edit_new)
-        popup_add (self.menu, "Edit...", self.edit_entry)
-        popup_add (self.menu, "Delete",  self.edit_delete)
+        self.menu_item = {}
+        self.menu_item ['add']    = popup_add (self.menu, "New...",  self.edit_new)
+        self.menu_item ['edit']   = popup_add (self.menu, "Edit...", self.edit_entry)
+        self.menu_item ['remove'] = popup_add (self.menu, "Delete",  self.edit_delete)
         self.menu.show ()
 
         colormap = self.get_colormap ()
@@ -159,7 +245,9 @@ class Pybliographic (GnomeApp):
         self.color ['blue'] = colormap.alloc ("blue")
 
         self.directory = './'
-        
+        self.modified  = 0
+        return
+    
     # --------------------------------------------------
         
     def quit (self, *arg):
@@ -168,16 +256,60 @@ class Pybliographic (GnomeApp):
     # --------------------------------------------------
 
     def file_new (self, *arg):
-        self.quit ()
+        win = URLFileSelection ("Save", FALSE, TRUE, FALSE)
+        win.set_filename (self.directory)
+
+        file = win.run ()
+        if file:
+            self.directory = os.path.split (file) [0] + '/'
+            self.new (file, win.type)
+        return
+
+
+    def new (self, file, type):
+        try:
+            self.database.set (Open.bibnew (file, type))
+            
+        except IOError, err:
+            
+            dialog = GnomeDialog ('Creation Error', STOCK_BUTTON_CLOSE)
+            dialog.button_connect (1, GnomeDialog.close)
+            dialog.set_usize (500, 300)
+            
+            text = GtkText ()
+            text.insert (None, None, None,
+                         "While creating, the following errors occured:\n\n")
+            text.insert (None, self.color ['red'], None, str (err))
+
+            holder = GtkScrolledWindow ()
+            holder.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
+            holder.add (text)
+
+            dialog.vbox.pack_start (holder)
+            
+            dialog.show_all ()
+            dialog.run_and_close ()
+            return
+
+        self.status.pop (1)
+        self.status.push (1, "Database `%s' with %d entries"
+                          % (file, len (self.database.data.keys ())))
+        return
     
     # --------------------------------------------------
 
-    def open (self, file):
+    def open (self, file, how = None):
         """ open a database """
+        
+        Utils.set_cursor (self, 'clock')
 
         try:
-            self.database.set (bibopen (file))
+            self.database.set (Open.bibopen (file, how))
+            Utils.set_cursor (self, 'normal')
+            
         except IOError, err:
+            Utils.set_cursor (self, 'normal')
+            
             dialog = GnomeDialog ('Open Error', STOCK_BUTTON_CLOSE)
             dialog.button_connect (1, GnomeDialog.close)
             dialog.set_usize (500, 300)
@@ -196,41 +328,60 @@ class Pybliographic (GnomeApp):
             dialog.show_all ()
             dialog.run_and_close ()
             return
-        
+
+        self.status.pop (1)
         self.status.push (1, "Database `%s' with %d entries"
                           % (file, len (self.database.data.keys ())))
 
+
     def file_open (self, *arg):
         """ callback for the Open button """
         
-        win = GtkExtra._FileSelection (TRUE)
-        win.set_title ("Open")
+        win = URLFileSelection ("Open", TRUE, TRUE)
         win.set_filename (self.directory)
-        
-        win.show()
-        mainloop()
 
-        file = win.ret
+        file = win.run ()
         
         if file:
             self.directory = os.path.split (file) [0] + '/'
-            self.open (file)
+            self.open (file, win.type)
 
         
     # --------------------------------------------------
 
     def file_exit (self, *arg):
-        self.quit ()
+        def callback (arg, self = self):
+            if arg == 0: self.modified = 0
+            return
+
+        if self.modified:
+            d = GnomeQuestionDialog ("Database is modified. Quit without saving ?", callback)
+            d.run_and_close ()
+            
+        if not self.modified: self.quit ()
+        return
+    
 
     # --------------------------------------------------
 
     def file_save (self, *arg):
-        pass
-
+        try:
+            self.database.data.update ()
+            self.status.pop (1)
+            self.status.push (1, "Database saved")
+            self.modified = 0
+            
+        except IOError, err:
+            GnomeErrorDialog ("can't save file: %s" % str (err)).run_and_close ()
+        return
+    
     # --------------------------------------------------
 
     def file_save_as (self, *arg):
-        pass
+        win = URLFileSelection ("Save", FALSE, TRUE)
+        win.set_filename (self.directory)
+
+        file = win.run ()
 
     # --------------------------------------------------
     # --------------------------------------------------
@@ -251,12 +402,34 @@ class Pybliographic (GnomeApp):
     # --------------------------------------------------
 
     def edit_search (self, * arg):
-        search = Search (self.database)
+        search = SearchDialog (self.database)
         search.show_all ()
-            
+        return
+    
     # --------------------------------------------------
 
     def edit_new (self, *arg):
+        selection = self.database.selection
+        
+        if not selection: 
+            row = 0
+        else:
+            row = self.database.selected_rows () [0]
+
+        entbase = Config.get ("base/entries").data.values () [0]
+        entry = Base.Entry (Base.Key (self.database.data, ''), '', entbase)
+        
+        def callback (entry, self, selection = row):
+            self.status.pop (1)
+            self.status.push (1, "...modified...")
+            self.modified = 1
+            entry.key = Base.Key (self.database.data, entry.name)
+            self.database.data [entry.key] = entry
+            self.database.insert (selection, entry)
+            return
+        
+        dialog = EntryDialog (entry, self.database.data, callback, self)
+        dialog.show_all ()
         pass
 
     # --------------------------------------------------
@@ -264,13 +437,21 @@ class Pybliographic (GnomeApp):
     def edit_entry (self, *arg):
         selection = self.database.selection
         if not selection: return
-        
+
+        row = self.database.selected_rows () [0]
         e = selection [0]
-        test = GtkWindow ()
-        ve = Entry (e)
-        ve.show ()
-        test.add (ve)
-        test.show ()
+        
+        def callback (entry, self, selection = row):
+            self.status.pop (1)
+            self.status.push (1, "...modified...")
+            self.modified = 1
+            
+            self.database.select_row (selection, 0)
+            self.database.set_row (selection, entry)
+            return
+        
+        dialog = EntryDialog (e, self.database.data, callback, self)
+        dialog.show_all ()
 
     # --------------------------------------------------
 
@@ -278,6 +459,23 @@ class Pybliographic (GnomeApp):
         selection = self.database.selection
         if not selection: return
 
+        entry = selection [0]
+        self.__ans = 0
+        def callback (arg, self = self):
+            if arg == 0: self.__ans = 1
+            return
+
+        d = GnomeQuestionDialog ("Really remove entry %s ?" % (entry.name), callback)
+        d.run_and_close ()
+
+        if self.__ans:
+            self.database.remove (entry.key)
+            self.modified = 1
+            self.status.pop (1)
+            self.status.push (1, "...modified...")
+        return
+    
+        
     # --------------------------------------------------
 
     def configure_prefs (self, * arg):
@@ -287,8 +485,11 @@ class Pybliographic (GnomeApp):
 
     def popup_menu (self, *arg):
         clist, event, = arg
-
+        if self.database.data is None: return
+        
         if (event.type == GDK._2BUTTON_PRESS and event.button == 1):
+            if not self.database.data.properties ['edit']: return
+            
             # Always select the item below the cursor
             couple = self.database.get_selection_info (event.x, event.y)
             if couple:
@@ -296,6 +497,9 @@ class Pybliographic (GnomeApp):
                 self.edit_entry (arg)
             
         if (event.type == GDK.BUTTON_PRESS and event.button == 3):
+            for f in ('add', 'edit', 'remove'):
+                self.menu_item [f].set_sensitive (self.database.data.properties [f])
+            
             # Always select the item below the cursor
             couple = self.database.get_selection_info (event.x, event.y)
             if couple:
@@ -359,3 +563,10 @@ class Pybliographic (GnomeApp):
                    ['Frdric Gobry'],
                    'Gnome interface to the Pybliographer system').show()
 
+
+    def key_handler (self, * arg):
+        event = arg [1]
+        if event.keyval == GDK.Return:
+             self.database.select_row (self.database.focus_row, 0)
+        return
+    
