@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Open.py,v 1.9 1999/07/30 12:32:13 gobry Exp $
+# $Id: Open.py,v 1.12 1999/08/18 14:05:16 gobry Exp $
 
 
 from types import *
@@ -32,12 +32,25 @@ import urlparse, urllib, traceback, os, sys, tempfile
 OpeningMethods = {}
 ListedMethods = []
 
-def register (name, opener, printer):
+def available ():
+	liste = ListedMethods + Autoload.available ()
+	hash  = {}
+	def _mapper (x, h = hash):
+		h [x] = 1
+		return 1
+	
+	map (_mapper, liste)
+	liste = hash.keys ()
+	liste.sort ()
+	
+	return liste
+
+def register (name, properties):
 	""
 	if OpeningMethods.has_key (name):
 		return
 	
-	OpeningMethods[name] = opener, printer
+	OpeningMethods [name] = properties
 	ListedMethods.append (name)
 
 
@@ -58,11 +71,15 @@ One can apply the following commands on the output of bibopen :
 """)
 
 
-def get (entity):
+def get (entity, method):
+	""" returns a specific field of the given entity """
+	
 	if OpeningMethods.has_key (entity):
-		return OpeningMethods [entity]
+		meth = OpeningMethods [entity]
+		if meth.has_key (method):
+			return meth [method]
 
-	return None, None
+	return None
 
 
 def bibopen (entity, how = None):
@@ -73,12 +90,12 @@ def bibopen (entity, how = None):
 
 		if how == None:
 			for method in ListedMethods:
-				opener = get (method) [0]
+				opener = get (method, 'open')
 				if opener:
 					base = opener (url, 1)
 					if base: break
 		else:
-			opener = get (how) [0]
+			opener = get (how, 'open')
 			if opener:
 				base = opener (url, 0)
 			else:
@@ -95,7 +112,10 @@ def bibopen (entity, how = None):
 		url [2] = os.path.expanduser (url [2])
 
 	# eventually load a new module
-	Autoload.check (url)
+	if how is None:
+		Autoload.check_by_url (url)
+	else:
+		Autoload.check_by_name (how)
 	
 	base = simple_try (url, how)
 	
@@ -134,9 +154,43 @@ def bibwrite (entity, out = None, how = None):
 
 	
 	if how == None:
-		writer = simplewrite
-	else:
-		writer = get (how) [1]
+		entity.foreach (simplewrite, out)
+		return
 
-	entity.foreach (writer, out)
+	Autoload.check_by_name (how)
+	writer = get (how, 'write')
 	
+	if writer is None:
+		raise IOError, "type `%s' does not specify write method" % how
+
+	writer (entity, out)
+	return
+
+Pyblio.Help.register ('bibnew', """
+Syntax: bib = bibnew (name, type)
+
+Creates a new bibliographic database of a given type
+""")
+
+def bibnew (name, type = None):
+
+	opener = get (type, 'new')
+	
+	if opener is None:
+		if os.path.exists (name):
+			raise IOError, "file `%s' exists" % name
+		
+		file = open (name, 'w')
+		file.close ()
+		
+		return bibopen (name, type)
+
+	# Consider the reference as an URL
+	url = list (urlparse.urlparse (name))
+
+	if url [0] == '':
+		# Consider we handle a local file
+		url [0] = 'file'
+		url [2] = os.path.expanduser (url [2])
+
+	return opener (url)
