@@ -18,25 +18,61 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: recodemodule.c,v 1.1 1999/07/22 07:10:26 gobry Exp $
+ $Id: recodemodule.c,v 1.4 1999/08/16 09:38:31 gobry Exp $
 */
 
-#include <stdbool.h>
-#include "Python.h"
-#include "recode.h"
+#ifdef HAVE_CONFIG_H
+#include "pyblioconf.h"
+#endif
 
+#ifdef HAVE_STDBOOL_H
+#include <stdbool.h>
+#else /* ! HAVE_STDBOOL_H */
+
+/* stdbool.h for GNU.  */
+
+/* The type `bool' must promote to `int' or `unsigned int'.  The constants
+   `true' and `false' must have the value 0 and 1 respectively.  */
+typedef enum
+  {
+    false = 0,
+    true = 1
+  } bool;
+
+/* The names `true' and `false' must also be made available as macros.  */
+#define false	false
+#define true	true
+
+/* Signal that all the definitions are present.  */
+#define __bool_true_false_are_defined	1
+
+#endif /* HAVE_STDBOOL_H */
+
+#include <stdio.h>
+
+#ifdef USE_RECODE
+#include <recode.h>
+#endif
+
+#include <Python.h>
+
+#ifdef USE_RECODE
 static RECODE_OUTER outer;
-char * program_name = "toto";
+char * program_name = "pyrecode";
+#endif
 
 static void
 py_delete_recoder (void * recoder) {
+#ifdef USE_RECODE
     RECODE_REQUEST request = (RECODE_REQUEST) recoder;
 
     recode_delete_request (request);
+#endif
 }
 
 static PyObject *
 py_new_recoder (PyObject * self, PyObject * args) {
+#ifdef USE_RECODE
     char * string;
     RECODE_REQUEST request;
 
@@ -55,10 +91,15 @@ py_new_recoder (PyObject * self, PyObject * args) {
     }
 
     return PyCObject_FromVoidPtr (request, py_delete_recoder);
+#else
+    Py_INCREF (Py_None);
+    return Py_None;
+#endif
 }
 
 static PyObject *
 py_recode (PyObject * self, PyObject * args) {
+#ifdef USE_RECODE
     char * string;
     RECODE_REQUEST request;
     PyObject * tmp;
@@ -78,6 +119,10 @@ py_recode (PyObject * self, PyObject * args) {
     free (string);
 
     return tmp;
+#else
+    Py_INCREF (Py_None);
+    return Py_None;
+#endif
 }
 
 static PyMethodDef recodeMeth [] = {
@@ -89,8 +134,13 @@ static PyMethodDef recodeMeth [] = {
 
 void initrecode ()
 {
+#ifdef USE_RECODE
     outer = recode_new_outer (false);
     if (outer == NULL) return;
+#else
+    fprintf (stderr, "warning: module recode is currently disabled.\n");
+    fprintf (stderr, "warning: to enable it, install GNU Recode and recompile.\n");
+#endif
 
     (void) Py_InitModule ("recode", recodeMeth);
 }
